@@ -1,18 +1,21 @@
 #include <sstream>
 #include <string>        // std::string, std::to_string
 #include <vector>        // for using vector
-#include "matrix.h"
 #include <limits>        // for std::numeric_limits<long double>::max()
-
+#include <R.h>
+#include <Rmath.h>
+#include "matrix.h"
 #include "rgwish.h"
 #include "copula.h"
 
 using namespace std;
 
 extern "C" {
-// Gaussian copula graphical models 
-// based on birth-death MCMC algorithm ***********************
- 
+/*
+ * birth-death MCMC for Gaussian copula Graphical models  
+ * with exact value of normalizing constant for D = I_p 
+ * it is for Bayesian model averaging
+*/
 void bdmcmcCopulap_links( int *iter, int *burnin, int G[], double Ts[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 double K_hat[], double p_links[],
@@ -42,7 +45,7 @@ void bdmcmcCopulap_links( int *iter, int *burnin, int G[], double Ts[], double K
 	double alpha = 1.0, beta = 0.0;
 	char transT = 'T', transN = 'N';																	
 
-	// Count  size of notes
+	// Count size of notes
 	int ip;
 	vector<int> size_node( dim, 0 );
 	for( i = 0; i < dim; i++ )
@@ -64,22 +67,22 @@ void bdmcmcCopulap_links( int *iter, int *burnin, int G[], double Ts[], double K
 			counter++;
 		}
 
-	vector<double> Kj12( p1 );             // K[j, -j]
-	vector<double> sigmaj12( p1 );         // sigma[-j, j]  
-	vector<double> sigmaj22( p1xp1 );      // sigma[-j, -j]
+	vector<double> Kj12( p1 );              // K[j, -j]
+	vector<double> sigmaj12( p1 );          // sigma[-j, j]  
+	vector<double> sigmaj22( p1xp1 );       // sigma[-j, -j]
 	vector<double> Kj22_inv( p1xp1 ); 
 	vector<double> Kj12xK22_inv( p1 ); 
 
-	vector<double> K12( p2x2 );            // K[e, -e]
-	vector<double> sigma11( 4 );           // sigma[e, e]
-	vector<double> sigma12( p2x2 );        // sigma[e, -e]
-	vector<double> sigma22( p2xp2 );       // sigma[-e, -e]
+	vector<double> K12( p2x2 );             // K[e, -e]
+	vector<double> sigma11( 4 );            // sigma[e, e]
+	vector<double> sigma12( p2x2 );         // sigma[e, -e]
+	vector<double> sigma22( p2xp2 );        // sigma[-e, -e]
 	vector<double> sigma11_inv( 4 ); 
 	vector<double> sigma21xsigma11_inv( p2x2 ); 
 	vector<double> sigma2112( p2xp2 ); 
 	vector<double> K22_inv( p2xp2 ); 
 	vector<double> K12xK22_inv( p2x2 );   
-// ---- for rgwish_sigma 
+	// ---- for rgwish_sigma 
 	vector<double> sigma_start( pxp ); 
 	vector<double> inv_C( pxp ); 
 	vector<double> beta_star( dim ); 
@@ -87,26 +90,26 @@ void bdmcmcCopulap_links( int *iter, int *burnin, int G[], double Ts[], double K
 	vector<double> sigma_start_N_i( dim );   // For dynamic memory used
 	vector<double> sigma_N_i( pxp );         // For dynamic memory used
 	vector<int> N_i( dim );                  // For dynamic memory used
-// ---- for copula ------------------------
+	// ---- for copula ------------------------
 	vector<double> S( pxp ); 
 	vector<double> inv_Ds( pxp ); 
 	vector<double> copy_Ds( pxp ); 
-// ----------------------------------------
+	// ----------------------------------------
 
 	long double max_numeric_limits_ld = std::numeric_limits<long double>::max() / 10000;
 
 	GetRNGstate();
+	// main loop for birth-death MCMC sampling algorithm ----------------------| 
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc++ )
 	{
 		if( ( i_mcmc + 1 ) % 1000 == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
 
-// Step1: copula
+		// STEP 1: copula -----------------------------------------------------|
 		get_Ds( K, Z, R, D, Ds, &S[0], gcgm, n, &dim );
 		get_Ts( Ds, Ts, &inv_Ds[0], &copy_Ds[0], &dim );
 		
-// Step2: computing birth and death rates
 		counter = 0;
-		
+		// STEP 2: calculating birth and death rates --------------------------|	
 		for( j = 1; j < dim; j++ )
 		{
 			jj   = j * dim + j;
@@ -128,18 +131,17 @@ void bdmcmcCopulap_links( int *iter, int *burnin, int G[], double Ts[], double K
 				ij   = j * dim + i;
 				Dsij = Ds[ij];
 
-	// For (i,j) = 0 ---------------------------------------------------------------|
-				sub_row_mins( K, &Kj12[0], &j, &dim );  // K12 = K[j, -j]  
-				Kj12[ i ] = 0.0;                      // K12[1,i] = 0
+				// For (i,j) = 0 ----------------------------------------------|
+				sub_row_mins( K, &Kj12[0], &j, &dim );   // K12 = K[j, -j]  
+				Kj12[ i ] = 0.0;                         // K12[1,i] = 0
 
 				// K12 %*% K22_inv
-				//multiply_matrix( &Kj12[0], &Kj22_inv[0], &Kj12xK22_inv[0], &one, &p1, &p1 );															
 				F77_NAME(dgemm)( &transN, &transN, &one, &p1, &p1, &alpha, &Kj12[0], &one, &Kj22_inv[0], &p1, &beta, &Kj12xK22_inv[0], &one );
 				
 				// K022  <- K_12 %*% solve( K0[-j, -j] ) %*% t(K_12)
 				F77_NAME(dgemm)( &transN, &transT, &one, &one, &p1, &alpha, &Kj12xK22_inv[0], &one, &Kj12[0], &one, &beta, &K022, &one );			
 
-	// For (i,j) = 1 ---------------------------------------------------------------|
+				// For (i,j) = 1 ----------------------------------------------|
 				sub_rows_mins( K, &K12[0], &i, &j, &dim );  // K12 = K[e, -e]  
 				
 				sub_matrices( &sigma[0], &sigma11[0], &sigma12[0], &sigma22[0], &i, &j, &dim );
@@ -151,42 +153,41 @@ void bdmcmcCopulap_links( int *iter, int *burnin, int G[], double Ts[], double K
 				F77_NAME(dgemm)( &transT, &transN, &p2, &two, &two, &alpha, &sigma12[0], &two, &sigma11_inv[0], &two, &beta, &sigma21xsigma11_inv[0], &p2 );
 
 				// sigma21xsigma11_inv %*% sigma12
-				//multiply_matrix( &sigma21xsigma11_inv[0], &sigma12[0], &sigma2112[0], &p2, &p2, &two );
 				F77_NAME(dgemm)( &transN, &transN, &p2, &p2, &two, &alpha, &sigma21xsigma11_inv[0], &p2, &sigma12[0], &two, &beta, &sigma2112[0], &p2 );
 
 				// solve( K[-e, -e] ) = sigma22 - sigma2112
-				for( k = 0; k < p2xp2 ; k++ ) K22_inv[k] = sigma22[k] - sigma2112[k];	
+				for( k = 0; k < p2xp2 ; k++ ) 
+					K22_inv[k] = sigma22[k] - sigma2112[k];	
 				
 				// K12 %*% K22_inv
-				// multiply_matrix( &K12[0], &K22_inv[0], &K12xK22_inv[0], &two, &p2, &p2 );
 				F77_NAME(dgemm)( &transN, &transN, &two, &p2, &p2, &alpha, &K12[0], &two, &K22_inv[0], &p2, &beta, &K12xK22_inv[0], &two );
 				
 				// K121 <- K[e, -e] %*% solve( K[-e, -e] ) %*% t(K[e, -e]) 															
 				F77_NAME(dgemm)( &transN, &transT, &two, &two, &p2, &alpha, &K12xK22_inv[0], &two, &K12[0], &two, &beta, &K121[0], &two );		
-	// Finished (i,j) = 1-----------------------------------------------------------|
+				// Finished (i,j) = 1------------------------------------------|
 
 				a11     = K[i * dim + i] - K121[0];	
 				sum_diag = - Dsij * K121[1] - Dsij * K121[2] + Dsjj * ( K022 - K121[3] );
 
-				//	nu_star = b + sum( Gf[,i] * Gf[,j] )
+				// nu_star = b + sum( Gf[,i] * Gf[,j] )
 				nu_star = b1;
 				for( k = 0; k < dim; k++ )
 					nu_star += G[i * dim + k] * G[j * dim + k];
 
-				//~ rate = sqrt( 2.0 * Dsjj / a11 ) * exp( lgamma( ( nu_star + 1 ) / 2 ) - lgamma( nu_star / 2 ) - ( Dsij * Dsij * a11 / Dsjj  + sum_diag ) / 2 );
 				rate = ( G[ij] ) 
 					? sqrt( Dsjj / a11 ) * exp( lgamma( ( nu_star + 1 ) / 2 ) - lgamma( nu_star / 2 ) - ( Dsij * Dsij * a11 / Dsjj  + sum_diag ) / 2 )
 					: sqrt( a11 / Dsjj ) * exp( lgamma( nu_star / 2 ) - lgamma( ( nu_star + 1 ) / 2 ) + ( Dsij * Dsij * a11 / Dsjj  + sum_diag ) / 2 );
 				
-				rates[counter++] = ( isinf( rate ) ) ? max_numeric_limits_ld : rate;
+				rates[counter++] = ( R_FINITE( rate ) ) ? rate : max_numeric_limits_ld;
 			}
 		}
 		
-		// select_edge( double rates[], int *index_selected_edge, double *qsume_rates, int *qp )
+		// Selecting an edge based on birth and death rates
 		select_edge( &rates[0], &index_selected_edge, &sum_rates, &qp );
 		selected_edge_i = index_rates_row[ index_selected_edge ];
 		selected_edge_j = index_rates_col[ index_selected_edge ];
 
+		// Updating G (graph) based on selected edge
 		selected_edge_ij    = selected_edge_j * dim + selected_edge_i;
 		G[selected_edge_ij] = 1 - G[selected_edge_ij];
 		G[selected_edge_i * dim + selected_edge_j] = G[selected_edge_ij];
@@ -202,10 +203,10 @@ void bdmcmcCopulap_links( int *iter, int *burnin, int G[], double Ts[], double K
 			--size_node[selected_edge_j]; 
 		}
 
-// STEP 3: Sampling from G-Wishart for new graph
+		// STEP 3: Sampling from G-Wishart for new graph ----------------------|	
 		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 
-// saving result --------------------------------------------------------------|	
+		// saving result ------------------------------------------------------|	
 		if( i_mcmc >= burn_in )
 		{
 			for( i = 0; i < pxp ; i++ )
@@ -215,9 +216,8 @@ void bdmcmcCopulap_links( int *iter, int *burnin, int G[], double Ts[], double K
 			}
 			
 			sum_weights += 1.0 / sum_rates;
-		}	
-// Finished saving result -----------------------------------------------------|	
-	}
+		} // End of saving result ---------------------------------------------|	
+	} // End of MCMC sampling algorithm ---------------------------------------| 
 	PutRNGstate();
 
 	for( i = 0; i < pxp; i++ )
@@ -227,6 +227,11 @@ void bdmcmcCopulap_links( int *iter, int *burnin, int G[], double Ts[], double K
 	}
 }
     
+/*
+ * birth-death MCMC for Gaussian copula Graphical models  
+ * with exact value of normalizing constant for D = I_p 
+ * it is for maximum a posterior probability estimation (MAP)
+*/
 void bdmcmcCopula( int *iter, int *burnin, int G[], double Ts[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 int all_graphs[], double all_weights[], double K_hat[], 
@@ -259,7 +264,7 @@ void bdmcmcCopula( int *iter, int *burnin, int G[], double Ts[], double K[], int
 	double alpha = 1.0, beta = 0.0;
 	char transT = 'T', transN = 'N';																	
 
-	// Count  size of notes
+	// Count size of notes
 	int ip;
 	vector<int> size_node( dim, 0 );
 	for( i = 0; i < dim; i++ )
@@ -281,22 +286,22 @@ void bdmcmcCopula( int *iter, int *burnin, int G[], double Ts[], double K[], int
 			counter++;
 		}
 
-	vector<double> Kj12( p1 );             // K[j, -j]
-	vector<double> sigmaj12( p1 );         // sigma[-j, j]  
-	vector<double> sigmaj22( p1xp1 );      // sigma[-j, -j]
+	vector<double> Kj12( p1 );               // K[j, -j]
+	vector<double> sigmaj12( p1 );           // sigma[-j, j]  
+	vector<double> sigmaj22( p1xp1 );        // sigma[-j, -j]
 	vector<double> Kj22_inv( p1xp1 ); 
 	vector<double> Kj12xK22_inv( p1 ); 
 
-	vector<double> K12( p2x2 );            // K[e, -e]
-	vector<double> sigma11( 4 );           // sigma[e, e]
-	vector<double> sigma12( p2x2 );        // sigma[e, -e]
-	vector<double> sigma22( p2xp2 );       // sigma[-e, -e]
+	vector<double> K12( p2x2 );              // K[e, -e]
+	vector<double> sigma11( 4 );             // sigma[e, e]
+	vector<double> sigma12( p2x2 );          // sigma[e, -e]
+	vector<double> sigma22( p2xp2 );         // sigma[-e, -e]
 	vector<double> sigma11_inv( 4 ); 
 	vector<double> sigma21xsigma11_inv( p2x2 ); 
 	vector<double> sigma2112( p2xp2 ); 
 	vector<double> K22_inv( p2xp2 ); 
 	vector<double> K12xK22_inv( p2x2 );   
-// ---- for rgwish_sigma 
+	// ---- for rgwish_sigma 
 	vector<double> sigma_start( pxp ); 
 	vector<double> inv_C( pxp ); 
 	vector<double> beta_star( dim ); 
@@ -304,26 +309,26 @@ void bdmcmcCopula( int *iter, int *burnin, int G[], double Ts[], double K[], int
 	vector<double> sigma_start_N_i( dim );   // For dynamic memory used
 	vector<double> sigma_N_i( pxp );         // For dynamic memory used
 	vector<int> N_i( dim );                  // For dynamic memory used
-// ---- for copula ------------------------
+	// ---- for copula ------------------------
 	vector<double> S( pxp ); 
 	vector<double> inv_Ds( pxp ); 
 	vector<double> copy_Ds( pxp ); 
-// ----------------------------------------
+	// ----------------------------------------
 
 	long double max_numeric_limits_ld = std::numeric_limits<long double>::max() / 10000;
 
 	GetRNGstate();
+	// main loop for birth-death MCMC sampling algorithm ----------------------| 
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc++ )
 	{
 		if( ( i_mcmc + 1 ) % 1000 == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
 
-// Step1: copula
+		// STEP 1: copula -----------------------------------------------------|
 		get_Ds( K, Z, R, D, Ds, &S[0], gcgm, n, &dim );
 		get_Ts( Ds, Ts, &inv_Ds[0], &copy_Ds[0], &dim );
 		
-// Step2: computing birth and death rates
 		counter = 0;
-
+		// STEP 2: calculating birth and death rates --------------------------|
 		for( j = 1; j < dim; j++ )
 		{
 			jj   = j * dim + j;
@@ -345,18 +350,17 @@ void bdmcmcCopula( int *iter, int *burnin, int G[], double Ts[], double K[], int
 				ij   = j * dim + i;
 				Dsij = Ds[ij];
 
-	// For (i,j) = 0 ---------------------------------------------------------------|
-				sub_row_mins( K, &Kj12[0], &j, &dim );  // K12 = K[j, -j]  
-				Kj12[ i ] = 0.0;                      // K12[1,i] = 0
+				// For (i,j) = 0 ----------------------------------------------|
+				sub_row_mins( K, &Kj12[0], &j, &dim );    // K12 = K[j, -j]  
+				Kj12[ i ] = 0.0;                          // K12[1,i] = 0
 
 				// K12 %*% K22_inv
-				//multiply_matrix( &Kj12[0], &Kj22_inv[0], &Kj12xK22_inv[0], &one, &p1, &p1 );															
 				F77_NAME(dgemm)( &transN, &transN, &one, &p1, &p1, &alpha, &Kj12[0], &one, &Kj22_inv[0], &p1, &beta, &Kj12xK22_inv[0], &one );
 				
 				// K022  <- K_12 %*% solve( K0[-j, -j] ) %*% t(K_12)
 				F77_NAME(dgemm)( &transN, &transT, &one, &one, &p1, &alpha, &Kj12xK22_inv[0], &one, &Kj12[0], &one, &beta, &K022, &one );			
 
-	// For (i,j) = 1 ---------------------------------------------------------------|
+				// For (i,j) = 1 ----------------------------------------------|
 				sub_rows_mins( K, &K12[0], &i, &j, &dim );  // K12 = K[e, -e]  
 				
 				sub_matrices( &sigma[0], &sigma11[0], &sigma12[0], &sigma22[0], &i, &j, &dim );
@@ -368,34 +372,32 @@ void bdmcmcCopula( int *iter, int *burnin, int G[], double Ts[], double K[], int
 				F77_NAME(dgemm)( &transT, &transN, &p2, &two, &two, &alpha, &sigma12[0], &two, &sigma11_inv[0], &two, &beta, &sigma21xsigma11_inv[0], &p2 );
 
 				// sigma21xsigma11_inv %*% sigma12
-				//multiply_matrix( &sigma21xsigma11_inv[0], &sigma12[0], &sigma2112[0], &p2, &p2, &two );
 				F77_NAME(dgemm)( &transN, &transN, &p2, &p2, &two, &alpha, &sigma21xsigma11_inv[0], &p2, &sigma12[0], &two, &beta, &sigma2112[0], &p2 );
 
 				// solve( K[-e, -e] ) = sigma22 - sigma2112
-				for( k = 0; k < p2xp2 ; k++ ) K22_inv[k] = sigma22[k] - sigma2112[k];	
+				for( k = 0; k < p2xp2 ; k++ ) 
+					K22_inv[k] = sigma22[k] - sigma2112[k];	
 				
 				// K12 %*% K22_inv
-				// multiply_matrix( &K12[0], &K22_inv[0], &K12xK22_inv[0], &two, &p2, &p2 );
 				F77_NAME(dgemm)( &transN, &transN, &two, &p2, &p2, &alpha, &K12[0], &two, &K22_inv[0], &p2, &beta, &K12xK22_inv[0], &two );
 				
 				// K121 <- K[e, -e] %*% solve( K[-e, -e] ) %*% t(K[e, -e]) 														
 				F77_NAME(dgemm)( &transN, &transT, &two, &two, &p2, &alpha, &K12xK22_inv[0], &two, &K12[0], &two, &beta, &K121[0], &two );		
-	// Finished (i,j) = 1-----------------------------------------------------------|
+				// Finished (i,j) = 1------------------------------------------|
 
 				a11     = K[i * dim + i] - K121[0];	
 				sum_diag = - Dsij * K121[1] - Dsij * K121[2] + Dsjj * ( K022 - K121[3] );
 
-				//	nu_star = b + sum( Gf[,i] * Gf[,j] )
+				// nu_star = b + sum( Gf[,i] * Gf[,j] )
 				nu_star = b1;
 				for( k = 0; k < dim; k++ )
 					nu_star += G[i * dim + k] * G[j * dim + k];
 
-				//~ rate = sqrt( 2.0 * Dsjj / a11 ) * exp( lgamma( ( nu_star + 1 ) / 2 ) - lgamma( nu_star / 2 ) - ( Dsij * Dsij * a11 / Dsjj  + sum_diag ) / 2 );
 				rate = ( G[ij] ) 
 					? sqrt( Dsjj / a11 ) * exp( lgamma( ( nu_star + 1 ) / 2 ) - lgamma( nu_star / 2 ) - ( Dsij * Dsij * a11 / Dsjj  + sum_diag ) / 2 )
 					: sqrt( a11 / Dsjj ) * exp( lgamma( nu_star / 2 ) - lgamma( ( nu_star + 1 ) / 2 ) + ( Dsij * Dsij * a11 / Dsjj  + sum_diag ) / 2 );
 				
-				rates[counter] = ( isinf( rate ) ) ? max_numeric_limits_ld : rate;
+				rates[counter] = ( R_FINITE( rate ) ) ? rate : max_numeric_limits_ld;
 				
 				char_g[counter] = G[ij] + '0'; 
 				counter++; 
@@ -403,11 +405,12 @@ void bdmcmcCopula( int *iter, int *burnin, int G[], double Ts[], double K[], int
 			}
 		}
 		
-		// select_edge( double rates[], int *index_selected_edge, double *qsume_rates, int *qp )
+		// Selecting an edge based on birth and death rates
 		select_edge( &rates[0], &index_selected_edge, &sum_rates, &qp );
 		selected_edge_i = index_rates_row[ index_selected_edge ];
 		selected_edge_j = index_rates_col[ index_selected_edge ];
 
+		// Updating G (graph) based on selected edge
 		selected_edge_ij    = selected_edge_j * dim + selected_edge_i;
 		G[selected_edge_ij] = 1 - G[selected_edge_ij];
 		G[selected_edge_i * dim + selected_edge_j] = G[selected_edge_ij];
@@ -423,10 +426,10 @@ void bdmcmcCopula( int *iter, int *burnin, int G[], double Ts[], double K[], int
 			--size_node[selected_edge_j]; 
 		}
 
-// STEP 3: Sampling from G-Wishart for new graph
+		// STEP 3: Sampling from G-Wishart for new graph ----------------------|	
 		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 
-// saving result --------------------------------------------------------------|	
+		// saving result ------------------------------------------------------|	
 		if( i_mcmc >= burn_in )
 		{
 			for( i = 0; i < pxp ; i++ ) K_hat[i] += K[i] / sum_rates;	
@@ -454,9 +457,8 @@ void bdmcmcCopula( int *iter, int *burnin, int G[], double Ts[], double K[], int
 			
 			counterallG++; 
 			sum_weights += 1.0 / sum_rates;
-		}
-// Finished saving result -----------------------------------------------------|	
-	}
+		} // End of saving result ---------------------------------------------|	
+	} // End of MCMC sampling algorithm ---------------------------------------| 
 	PutRNGstate();
 
 	for( i = 0; i < ( iteration - burn_in ); i++ ) 
@@ -471,6 +473,6 @@ void bdmcmcCopula( int *iter, int *burnin, int G[], double Ts[], double K[], int
 		K_hat[i] /= sum_weights;
 }
        
-} // exturn "C"
+} // End of exturn "C"
 
 
