@@ -2,7 +2,8 @@
 #include <string>        // std::string, std::to_string
 #include <vector>        // for using vector
 #include <limits>        // for std::numeric_limits<long double>::max()
-
+#include <R.h>
+#include <Rmath.h>
 #include "matrix.h"
 #include "rgwish.h"
 
@@ -10,9 +11,11 @@ using namespace std;
 
 extern "C" {
 
-////////////////////////////////////////////////////////////////////////////////
-// bdmcmc algoirthm for GGMs with exact value of normalizing constant for D = I_p
-////////////////////////////////////////////////////////////////////////////////       
+/*
+ * Multiple birth-death MCMC for Gaussian Graphical models  
+ * with exact value of normalizing constant for D = I_p 
+ * it is for Bayesian model averaging
+*/
 void bdmcmcExactp_links_multi_update( int *iter, int *burnin, int G[], double Ts[], double K[], int *p, 
 			 double K_hat[], double p_links[],
 			 int *b, int *b_star, double Ds[], double *threshold, int *multi_update )
@@ -40,7 +43,7 @@ void bdmcmcExactp_links_multi_update( int *iter, int *burnin, int G[], double Ts
 	double alpha = 1.0, beta = 0.0;
 	char transT = 'T', transN = 'N';																	
 
-	// Count  size of notes
+	// Count size of notes
 	int ip;
 	vector<int> size_node( dim, 0 );
 	for( i = 0; i < dim; i++ )
@@ -63,21 +66,21 @@ void bdmcmcExactp_links_multi_update( int *iter, int *burnin, int G[], double Ts
 		}
 
 	vector<double> K121( 4 ); 
-	vector<double> Kj12( p1 );             // K[j, -j]
-	vector<double> sigmaj12( p1 );         // sigma[-j, j]  
-	vector<double> sigmaj22( p1xp1 );      // sigma[-j, -j]
+	vector<double> Kj12( p1 );              // K[j, -j]
+	vector<double> sigmaj12( p1 );          // sigma[-j, j]  
+	vector<double> sigmaj22( p1xp1 );       // sigma[-j, -j]
 	vector<double> Kj22_inv( p1xp1 ); 
 	vector<double> Kj12xK22_inv( p1 ); 
-	vector<double> K12( p2x2 );            // K[e, -e]
-	vector<double> sigma11( 4 );           // sigma[e, e]
-	vector<double> sigma12( p2x2 );        // sigma[e, -e]
-	vector<double> sigma22( p2xp2 );       // sigma[-e, -e]
+	vector<double> K12( p2x2 );             // K[e, -e]
+	vector<double> sigma11( 4 );            // sigma[e, e]
+	vector<double> sigma12( p2x2 );         // sigma[e, -e]
+	vector<double> sigma22( p2xp2 );        // sigma[-e, -e]
 	vector<double> sigma11_inv( 4 ); 
 	vector<double> sigma21xsigma11_inv( p2x2 ); 
 	vector<double> sigma2112( p2xp2 ); 
 	vector<double> K22_inv( p2xp2 ); 
 	vector<double> K12xK22_inv( p2x2 );   
-// ---- for rgwish_sigma 
+	// ---- for rgwish_sigma 
 	vector<double> sigma_start( pxp ); 
 	vector<double> inv_C( pxp ); 
 	vector<double> beta_star( dim ); 
@@ -85,7 +88,7 @@ void bdmcmcExactp_links_multi_update( int *iter, int *burnin, int G[], double Ts
 	vector<double> sigma_start_N_i( dim );   // For dynamic memory used
 	vector<double> sigma_N_i( pxp );         // For dynamic memory used
 	vector<int> N_i( dim );                  // For dynamic memory used
-// ----------------------------
+	// ----------------------------
 
 	int size_index = multi_update_C;
 	vector<int> index_selected_edges( multi_update_C );
@@ -93,13 +96,13 @@ void bdmcmcExactp_links_multi_update( int *iter, int *burnin, int G[], double Ts
 	long double max_numeric_limits_ld = std::numeric_limits<long double>::max() / 10000;
 
 	GetRNGstate();
+	// main loop for multiple birth-death MCMC sampling algorithm -------------| 
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc += size_index )
 	{
 		if( ( i_mcmc + 1 ) % 1000 == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
 		
-// STEP 1: calculating birth-death rates
 		counter = 0;
-		
+		// STEP 1: calculating birth and death rates --------------------------|			
 		for( j = 1; j < dim; j++ )
 		{
 			jj   = j * dim + j;
@@ -121,19 +124,18 @@ void bdmcmcExactp_links_multi_update( int *iter, int *burnin, int G[], double Ts
 				ij   = j * dim + i;
 				Dsij = Ds[ij];
 
-	// For (i,j) = 0 ---------------------------------------------------------------|	
-				sub_row_mins( K, &Kj12[0], &j, &dim );  // K12 = K[j, -j]  
-				Kj12[ i ] = 0.0;                      // K12[1,i] = 0
+				// For (i,j) = 0 ----------------------------------------------|	
+				sub_row_mins( K, &Kj12[0], &j, &dim );    // K12 = K[j, -j]  
+				Kj12[ i ] = 0.0;                          // K12[1,i] = 0
 
 				// K12 %*% K22_inv
-				//multiply_matrix( &Kj12[0], &Kj22_inv[0], &Kj12xK22_inv[0], &one, &p1, &p1 );															
 				F77_NAME(dgemm)( &transN, &transN, &one, &p1, &p1, &alpha, &Kj12[0], &one, &Kj22_inv[0], &p1, &beta, &Kj12xK22_inv[0], &one );
 				
 				// K022  <- K_12 %*% solve( K0[-j, -j] ) %*% t(K_12)
 				F77_NAME(dgemm)( &transN, &transT, &one, &one, &p1, &alpha, &Kj12xK22_inv[0], &one, &Kj12[0], &one, &beta, &K022, &one );			
 
-	// For (i,j) = 1 ---------------------------------------------------------------|
-				sub_rows_mins( K, &K12[0], &i, &j, &dim );  // K12 = K[e, -e]  
+				// For (i,j) = 1 ----------------------------------------------|
+				sub_rows_mins( K, &K12[0], &i, &j, &dim );   // K12 = K[e, -e]  
 				
 				sub_matrices( &sigma[0], &sigma11[0], &sigma12[0], &sigma22[0], &i, &j, &dim );
 
@@ -144,39 +146,38 @@ void bdmcmcExactp_links_multi_update( int *iter, int *burnin, int G[], double Ts
 				F77_NAME(dgemm)( &transT, &transN, &p2, &two, &two, &alpha, &sigma12[0], &two, &sigma11_inv[0], &two, &beta, &sigma21xsigma11_inv[0], &p2 );
 
 				// sigma21xsigma11_inv %*% sigma12
-				//multiply_matrix( &sigma21xsigma11_inv[0], &sigma12[0], &sigma2112[0], &p2, &p2, &two );
 				F77_NAME(dgemm)( &transN, &transN, &p2, &p2, &two, &alpha, &sigma21xsigma11_inv[0], &p2, &sigma12[0], &two, &beta, &sigma2112[0], &p2 );
 
 				// solve( K[-e, -e] ) = sigma22 - sigma2112
-				for( k = 0; k < p2xp2 ; k++ ) K22_inv[k] = sigma22[k] - sigma2112[k];	
+				for( k = 0; k < p2xp2 ; k++ ) 
+					K22_inv[k] = sigma22[k] - sigma2112[k];	
 				
 				// K12 %*% K22_inv
-				// multiply_matrix( &K12[0], &K22_inv[0], &K12xK22_inv[0], &two, &p2, &p2 );
 				F77_NAME(dgemm)( &transN, &transN, &two, &p2, &p2, &alpha, &K12[0], &two, &K22_inv[0], &p2, &beta, &K12xK22_inv[0], &two );
 				
 				// K121 <- K[e, -e] %*% solve( K[-e, -e] ) %*% t(K[e, -e]) 														
 				F77_NAME(dgemm)( &transN, &transT, &two, &two, &p2, &alpha, &K12xK22_inv[0], &two, &K12[0], &two, &beta, &K121[0], &two );		
-	// Finished (i,j) = 1-----------------------------------------------------------|
+				// Finished (i,j) = 1------------------------------------------|
 
-				a11     = K[i * dim + i] - K121[0];	
+				a11      = K[i * dim + i] - K121[0];	
 				sum_diag = - Dsij * K121[1] - Dsij * K121[2] + Dsjj * ( K022 - K121[3] );
 
-				//	nu_star = b + sum( Gf[,i] * Gf[,j] )
+				// nu_star = b + sum( Gf[,i] * Gf[,j] )
 				nu_star = b1;
 				for( k = 0; k < dim; k++ ) nu_star += G[i * dim + k] * G[j * dim + k];
 
-				//~ rate = sqrt( 2.0 * Dsjj / a11 ) * exp( lgamma( ( nu_star + 1 ) / 2 ) - lgamma( nu_star / 2 ) - ( Dsij * Dsij * a11 / Dsjj  + sum_diag ) / 2 );
 				rate = ( G[ij] ) 
 					? sqrt( Dsjj / a11 ) * exp( lgamma( ( nu_star + 1 ) / 2 ) - lgamma( nu_star / 2 ) - ( Dsij * Dsij * a11 / Dsjj  + sum_diag ) / 2 )
 					: sqrt( a11 / Dsjj ) * exp( lgamma( nu_star / 2 ) - lgamma( ( nu_star + 1 ) / 2 ) + ( Dsij * Dsij * a11 / Dsjj  + sum_diag ) / 2 );
 				
-				rates[counter++] = ( isinf( rate ) ) ? max_numeric_limits_ld : rate;
+				rates[counter++] = ( R_FINITE( rate ) ) ? rate : max_numeric_limits_ld;
 			}
 		}	
 
-		//select_multi_edges( double rates[], int index_selected_edges[], double *sum_rates, int *multi_update, int *qp )
+		// Selecting multiple edges based on birth and death rates
 		select_multi_edges( &rates[0], &index_selected_edges[0], &size_index, &sum_rates, &multi_update_C, &qp );
 
+		// Updating graph based on selected edges
 		for ( i = 0; i < size_index; i++ )
 		{
 			selected_edge_i = index_rates_row[ index_selected_edges[i] ];
@@ -198,10 +199,10 @@ void bdmcmcExactp_links_multi_update( int *iter, int *burnin, int G[], double Ts
 			}		
 		}
 
-// STEP 2: Sampling from G-Wishart for new graph
+		// STEP 2: Sampling from G-Wishart for new graph ----------------------|	
 		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 
-// saving result --------------------------------------------------------------|	
+		// saving result ------------------------------------------------------|	
 		if( i_mcmc >= burn_in )
 		{
 			for( i = 0; i < pxp ; i++ )
@@ -211,9 +212,8 @@ void bdmcmcExactp_links_multi_update( int *iter, int *burnin, int G[], double Ts
 			}
 			
 			sum_weights += 1.0 / sum_rates;
-		}	
-// Finished saving result -----------------------------------------------------|	
-	}
+		} // End of saving result ---------------------------------------------|	
+	} // End of MCMC sampling algorithm ---------------------------------------|
 	PutRNGstate();
 
 	for( i = 0; i < pxp; i++ )
@@ -223,6 +223,11 @@ void bdmcmcExactp_links_multi_update( int *iter, int *burnin, int G[], double Ts
 	}
 }
     
+/*
+ * Multiple birth-death MCMC for Gaussian Graphical models  
+ * with exact value of normalizing constant for D = I_p 
+ * it is for maximum a posterior probability estimation (MAP)
+*/
 void bdmcmcExact_multi_update( int *iter, int *burnin, int G[], double Ts[], double K[], int *p, 
 			 int all_graphs[], double all_weights[], double K_hat[], 
 			 char *sample_graphs[], double graph_weights[], int *size_sample_g,
@@ -277,21 +282,21 @@ void bdmcmcExact_multi_update( int *iter, int *burnin, int G[], double Ts[], dou
 		}
 
 	vector<double> K121( 4 ); 
-	vector<double> Kj12( p1 );             // K[j, -j]
-	vector<double> sigmaj12( p1 );         // sigma[-j, j]  
-	vector<double> sigmaj22( p1xp1 );      // sigma[-j, -j]
+	vector<double> Kj12( p1 );              // K[j, -j]
+	vector<double> sigmaj12( p1 );          // sigma[-j, j]  
+	vector<double> sigmaj22( p1xp1 );       // sigma[-j, -j]
 	vector<double> Kj22_inv( p1xp1 ); 
 	vector<double> Kj12xK22_inv( p1 ); 
-	vector<double> K12( p2x2 );            // K[e, -e]
-	vector<double> sigma11( 4 );           // sigma[e, e]
-	vector<double> sigma12( p2x2 );        // sigma[e, -e]
-	vector<double> sigma22( p2xp2 );       // sigma[-e, -e]
+	vector<double> K12( p2x2 );             // K[e, -e]
+	vector<double> sigma11( 4 );            // sigma[e, e]
+	vector<double> sigma12( p2x2 );         // sigma[e, -e]
+	vector<double> sigma22( p2xp2 );        // sigma[-e, -e]
 	vector<double> sigma11_inv( 4 ); 
 	vector<double> sigma21xsigma11_inv( p2x2 ); 
 	vector<double> sigma2112( p2xp2 ); 
 	vector<double> K22_inv( p2xp2 ); 
 	vector<double> K12xK22_inv( p2x2 );   
-// ---- for rgwish_sigma 
+	// ---- for rgwish_sigma 
 	vector<double> sigma_start( pxp ); 
 	vector<double> inv_C( pxp ); 
 	vector<double> beta_star( dim ); 
@@ -299,7 +304,7 @@ void bdmcmcExact_multi_update( int *iter, int *burnin, int G[], double Ts[], dou
 	vector<double> sigma_start_N_i( dim );   // For dynamic memory used
 	vector<double> sigma_N_i( pxp );         // For dynamic memory used
 	vector<int> N_i( dim );                  // For dynamic memory used
-// ----------------------------
+	// ----------------------------
 
 	int size_index = multi_update_C;
 	vector<int> index_selected_edges( multi_update_C );
@@ -307,13 +312,13 @@ void bdmcmcExact_multi_update( int *iter, int *burnin, int G[], double Ts[], dou
 	long double max_numeric_limits_ld = std::numeric_limits<long double>::max() / 10000;
 
 	GetRNGstate();
+	// main loop for multiple birth-death MCMC sampling algorithm -------------| 
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc += size_index )
 	{
 		if( ( i_mcmc + 1 ) % 1000 == 0 ) Rprintf( " Iteration  %d                 \n", i_mcmc + 1 ); 
 		
-// STEP 1: calculating birth-death rates
 		counter = 0;
-		
+		// STEP 1: calculating birth and death rates
 		for( j = 1; j < dim; j++ )
 		{
 			jj   = j * dim + j;
@@ -335,19 +340,18 @@ void bdmcmcExact_multi_update( int *iter, int *burnin, int G[], double Ts[], dou
 				ij   = j * dim + i;
 				Dsij = Ds[ij];
 
-	// For (i,j) = 0 ---------------------------------------------------------------|	
-				sub_row_mins( K, &Kj12[0], &j, &dim );  // K12 = K[j, -j]  
-				Kj12[ i ] = 0.0;                      // K12[1,i] = 0
+				// For (i,j) = 0 ----------------------------------------------|	
+				sub_row_mins( K, &Kj12[0], &j, &dim );   // K12 = K[j, -j]  
+				Kj12[ i ] = 0.0;                         // K12[1,i] = 0
 
 				// Kj12 %*% Kj22_inv
-				//multiply_matrix( &Kj12[0], &Kj22_inv[0], &Kj12xK22_inv[0], &one, &p1, &p1 );															
 				F77_NAME(dgemm)( &transN, &transN, &one, &p1, &p1, &alpha, &Kj12[0], &one, &Kj22_inv[0], &p1, &beta, &Kj12xK22_inv[0], &one );
 				
 				// K022  <- Kj12 %*% solve( K0[-j, -j] ) %*% t(Kj12)
 				F77_NAME(dgemm)( &transN, &transT, &one, &one, &p1, &alpha, &Kj12xK22_inv[0], &one, &Kj12[0], &one, &beta, &K022, &one );			
 
-	// For (i,j) = 1 ---------------------------------------------------------------|
-				sub_rows_mins( K, &K12[0], &i, &j, &dim );  // K12 = K[e, -e]  
+				// For (i,j) = 1 ----------------------------------------------|
+				sub_rows_mins( K, &K12[0], &i, &j, &dim );   // K12 = K[e, -e]  
 				
 				sub_matrices( &sigma[0], &sigma11[0], &sigma12[0], &sigma22[0], &i, &j, &dim );
 
@@ -358,42 +362,41 @@ void bdmcmcExact_multi_update( int *iter, int *burnin, int G[], double Ts[], dou
 				F77_NAME(dgemm)( &transT, &transN, &p2, &two, &two, &alpha, &sigma12[0], &two, &sigma11_inv[0], &two, &beta, &sigma21xsigma11_inv[0], &p2 );
 
 				// sigma21xsigma11_inv %*% sigma12
-				//multiply_matrix( &sigma21xsigma11_inv[0], &sigma12[0], &sigma2112[0], &p2, &p2, &two );
 				F77_NAME(dgemm)( &transN, &transN, &p2, &p2, &two, &alpha, &sigma21xsigma11_inv[0], &p2, &sigma12[0], &two, &beta, &sigma2112[0], &p2 );
 
 				// solve( K[-e, -e] ) = sigma22 - sigma2112
-				for( k = 0; k < p2xp2 ; k++ ) K22_inv[k] = sigma22[k] - sigma2112[k];	
+				for( k = 0; k < p2xp2 ; k++ ) 
+					K22_inv[k] = sigma22[k] - sigma2112[k];	
 				
 				// K12 %*% K22_inv
-				// multiply_matrix( &K12[0], &K22_inv[0], &K12xK22_inv[0], &two, &p2, &p2 );
 				F77_NAME(dgemm)( &transN, &transN, &two, &p2, &p2, &alpha, &K12[0], &two, &K22_inv[0], &p2, &beta, &K12xK22_inv[0], &two );
 				
 				// K121 <- K[e, -e] %*% solve( K[-e, -e] ) %*% t(K[e, -e]) 														
 				F77_NAME(dgemm)( &transN, &transT, &two, &two, &p2, &alpha, &K12xK22_inv[0], &two, &K12[0], &two, &beta, &K121[0], &two );		
-	// Finished (i,j) = 1-----------------------------------------------------------|
+				// Finished (i,j) = 1------------------------------------------|
 
-				a11     = K[i * dim + i] - K121[0];	
+				a11      = K[i * dim + i] - K121[0];	
 				sum_diag = - Dsij * K121[1] - Dsij * K121[2] + Dsjj * ( K022 - K121[3] );
 
-				//	nu_star = b + sum( Gf[,i] * Gf[,j] )
+				// nu_star = b + sum( Gf[,i] * Gf[,j] )
 				nu_star = b1;
 				for( k = 0; k < dim; k++ ) nu_star += G[i * dim + k] * G[j * dim + k];   
 
-				//~ rate = sqrt( 2.0 * Dsjj / a11 ) * exp( lgamma( ( nu_star + 1 ) / 2 ) - lgamma( nu_star / 2 ) - ( Dsij * Dsij * a11 / Dsjj  + sum_diag ) / 2 );
 				rate = ( G[ij] ) 
 					? sqrt( Dsjj / a11 ) * exp( lgamma( ( nu_star + 1 ) / 2 ) - lgamma( nu_star / 2 ) - ( Dsij * Dsij * a11 / Dsjj  + sum_diag ) / 2 )
 					: sqrt( a11 / Dsjj ) * exp( lgamma( nu_star / 2 ) - lgamma( ( nu_star + 1 ) / 2 ) + ( Dsij * Dsij * a11 / Dsjj  + sum_diag ) / 2 );
 				
-				rates[counter] = ( isinf( rate ) ) ? max_numeric_limits_ld : rate;
+				rates[counter] = ( R_FINITE( rate ) ) ? rate : max_numeric_limits_ld;
 				
 				char_g[counter] = G[ij] + '0'; 
 				counter++; 
 			}
 		}	
 			
-		//select_multi_edges( double rates[], int index_selected_edges[], double *sum_rates, int *multi_update, int *qp )
+		// Selecting multiple edges based on birth and death rates
 		select_multi_edges( &rates[0], &index_selected_edges[0], &size_index, &sum_rates, &multi_update_C, &qp );
 
+		// Updating graph based on selected edges
 		for ( i = 0; i < size_index; i++ )
 		{
 			selected_edge_i = index_rates_row[ index_selected_edges[i] ];
@@ -415,10 +418,10 @@ void bdmcmcExact_multi_update( int *iter, int *burnin, int G[], double Ts[], dou
 			}		
 		}
 
-// STEP 2: Sampling from G-Wishart for new graph
+		// STEP 2: Sampling from G-Wishart for new graph ----------------------|
 		rgwish_sigma( G, &size_node[0], Ts, K, &sigma[0], b_star, &dim, &threshold_C, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 
-// saving result --------------------------------------------------------------|	
+		// saving result ------------------------------------------------------|	
 		if( i_mcmc >= burn_in )
 		{
 			for( i = 0; i < pxp ; i++ ) K_hat[i] += K[i] / sum_rates;	
@@ -446,9 +449,8 @@ void bdmcmcExact_multi_update( int *iter, int *burnin, int G[], double Ts[], dou
 			
 			counterallG++; 
 			sum_weights += 1.0 / sum_rates;
-		}
-// Finished saving result -----------------------------------------------------|	
-	}
+		} // End of saving result ---------------------------------------------|	
+	} // End of MCMC sampling algorithm ---------------------------------------| 
 	PutRNGstate();
 
 	for( i = 0; i < ( iteration - burn_in ); i++ ) 
@@ -462,4 +464,4 @@ void bdmcmcExact_multi_update( int *iter, int *burnin, int G[], double Ts[], dou
 	for( i = 0; i < pxp; i++ ) K_hat[i] /= sum_weights;		
 }
               
-} // exturn "C"
+} // End of exturn "C"
